// -*- C++ -*-
/*!
 * @file  RTC_Kinect_Calibration.h
 * @brief ModuleDescription
 * @date  $Date$
 *
 * $Id$
 */

#ifndef RTC_KINECT_CALIBRATION_H
#define RTC_KINECT_CALIBRATION_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

#include "Kinect_CalibrationMain.h"

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>

using namespace RTC;

/*!
 * @class RTC_Kinect_Calibration
 * @brief ModuleDescription
 *
 */
class RTC_Kinect_Calibration
	: public RTC::DataFlowComponentBase {
public:
	/*!
	 * @brief constructor
	 * @param manager Maneger Object
	 */
	RTC_Kinect_Calibration(RTC::Manager* manager);

	/*!
	 * @brief destructor
	 */
	~RTC_Kinect_Calibration();

	// <rtc-template block="public_attribute">
  
	// </rtc-template>

	// <rtc-template block="public_operation">
  
	// </rtc-template>

	/***
	 *
	 * The initialize action (on CREATED->ALIVE transition)
	 * formaer rtc_init_entry()
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	virtual RTC::ReturnCode_t onInitialize();

	/***
	 *
	 * The finalize action (on ALIVE->END transition)
	 * formaer rtc_exiting_entry()
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onFinalize();

	/***
	 *
	 * The startup action when ExecutionContext startup
	 * former rtc_starting_entry()
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

	/***
	 *
	 * The shutdown action when ExecutionContext stop
	 * former rtc_stopping_entry()
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

	/***
	 *
	 * The activated action (Active state entry action)
	 * former rtc_active_entry()
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

	/***
	 *
	 * The deactivated action (Active state exit action)
	 * former rtc_active_exit()
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

	/***
	 *
	 * The execution action that is invoked periodically
	 * former rtc_active_do()
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

	/***
	 *
	 * The aborting action when main logic error occurred.
	 * former rtc_aborting_entry()
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

	/***
	 *
	 * The error action in ERROR state
	 * former rtc_error_do()
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

	/***
	 *
	 * The reset action that is invoked resetting
	 * This is same but different the former rtc_init_entry()
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

	/***
	 *
	 * The state update action that is invoked after onExecute() action
	 * no corresponding operation exists in OpenRTm-aist-0.2.0
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

	/***
	 *
	 * The action that is invoked when execution context's rate is changed
	 * no corresponding operation exists in OpenRTm-aist-0.2.0
	 *
	 * @param ec_id target ExecutionContext Id
	 *
	 * @return RTC::ReturnCode_t
	 *
	 *
	 */
	// virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


protected:
	// <rtc-template block="protected_attribute">
  
	// </rtc-template>

	// <rtc-template block="protected_operation">
  
	// </rtc-template>

	// Configuration variable declaration
	// <rtc-template block="config_declare">

	// </rtc-template>

	// DataInPort declaration
	// <rtc-template block="inport_declare">
  RTC::Point3D m_startPoint;
  /*!
   */
  InPort<RTC::Point3D> m_startPointIn;
  RTC::Point3D m_endPoint;
  /*!
   */
  InPort<RTC::Point3D> m_endPointIn;
  RTC::Point3D m_userPosition;
  /*!
   */
  InPort<RTC::Point3D> m_userPositionIn;
  
	// </rtc-template>


	// DataOutPort declaration
	// <rtc-template block="outport_declare">
  
	// </rtc-template>

	// CORBA Port declaration
	// <rtc-template block="corbaport_declare">
  
	// </rtc-template>

	// Service declaration
	// <rtc-template block="service_declare">
  
	// </rtc-template>

	// Consumer declaration
	// <rtc-template block="consumer_declare">
  
	// </rtc-template>

private:
	// <rtc-template block="private_attribute">
  
	// </rtc-template>

	// <rtc-template block="private_operation">
  
	// </rtc-template>

	KinectCalibrationMain calibrationMain;

};


extern "C"
{
	DLL_EXPORT void RTC_Kinect_CalibrationInit(RTC::Manager* manager);
};

#endif // RTC_KINECT_CALIBRATION_H
