// -*- C++ -*-
/*!
 * @file  RTC_Kinect_Calibration.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "RTC_Kinect_Calibration.h"

// Module specification
// <rtc-template block="module_spec">
static const char* rtc_kinect_calibration_spec[] =
  {
    "implementation_id", "RTC_Kinect_Calibration",
    "type_name",         "RTC_Kinect_Calibration",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "ryo takeuchi",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTC_Kinect_Calibration::RTC_Kinect_Calibration(RTC::Manager* manager)
// <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_startPointIn("dp_start", m_startPoint),
    m_endPointIn("dp_end", m_endPoint),
    m_userPositionIn("dp_user", m_userPosition)

// </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_Kinect_Calibration::~RTC_Kinect_Calibration()
{
}



RTC::ReturnCode_t RTC_Kinect_Calibration::onInitialize()
{
	// Registration: InPort/OutPort/Service
	// <rtc-template block="registration">
  // Set InPort buffers
  addInPort("dp_start", m_startPointIn);
  addInPort("dp_end", m_endPointIn);
  addInPort("dp_user", m_userPositionIn);
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
	// </rtc-template>

	// <rtc-template block="bind_config">
	// </rtc-template>

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onFinalize()
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onStartup(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onShutdown(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RTC_Kinect_Calibration::onActivated(RTC::UniqueId ec_id)
{
	calibrationMain.initialize();

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onDeactivated(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RTC_Kinect_Calibration::onExecute(RTC::UniqueId ec_id)
{
	if(m_startPointIn.isNew()) {
		m_startPointIn.read();
	}
	if(m_endPointIn.isNew()) {
		m_endPointIn.read();
	}
	if(m_userPositionIn.isNew()) {
		m_userPositionIn.read();
	}

	calibrationMain.update(cv::Point3d(m_startPoint.x, m_startPoint.y, m_startPoint.z),
		cv::Point3d(m_endPoint.x, m_endPoint.y, m_endPoint.z));

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onAborting(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onError(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onReset(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onStateUpdate(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Calibration::onRateChanged(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/



extern "C"
{

	void RTC_Kinect_CalibrationInit(RTC::Manager* manager)
	{
		coil::Properties profile(rtc_kinect_calibration_spec);
		manager->registerFactory(profile,
			RTC::Create<RTC_Kinect_Calibration>,
			RTC::Delete<RTC_Kinect_Calibration>);
	}

};


