#include "KinectSensor.h"

KinectSensor::KinectSensor(NUI_IMAGE_RESOLUTION cameraResolution, NUI_IMAGE_RESOLUTION depthResolution, int kinectID = 0){
	disableRGB = false;
	disableDepth = false;

	cv::Size screenSize;
	if(cameraResolution == NUI_IMAGE_RESOLUTION_1280x960) {
		screenSize = cv::Size(1280, 960);
	}
	else screenSize = cv::Size(640, 480);

	cv::Mat black_img = cv::Mat::zeros(screenSize, CV_8UC3);
	frame = black_img.clone();

	this->depthResolution = depthResolution;
	this->cameraResolution = cameraResolution;

	if(cameraResolution == NUI_IMAGE_RESOLUTION_640x480) {
		this->width = screenSize.width;
		this->height = screenSize.height;
	}
	else {
		this->width = screenSize.width;
		this->height = screenSize.height;
	}

	std::cout << "kinect initializing...";

	//Kinect̐擾
	int count = 0;
	try {
		::NuiGetSensorCount(&count);
	} catch(const char* str) {
		throw std::runtime_error("KinectڑĂ");
	}
	if(count == 0) {
		throw std::runtime_error("KinectڑĂ");
	}

	//CX^X쐬
	if(kinectID < count) {
		try {
			::NuiCreateSensorByIndex(kinectID, &kinect); //Kinectgpꍇ͂܂
		} catch(const char* str) {
			throw std::runtime_error(str);
		}
	}
	else {
		throw std::runtime_error("ȂKinect܂");
	}


	//kinect->NuiSetForceInfraredEmitterOff(TRUE); //ԊOOFF

	//Kinect̏Ԃ擾
	HRESULT status = kinect->NuiStatus();

	if(status != S_OK) {
		throw std::runtime_error("Kinectp\ł͂܂");
	}

	std::cout << "done." << std::endl;

	//Xg̃TCYύX
	//fɑ΂[xl𓾂邽߁ARGB̉𑜓xŊmۂ
	this->depth.resize(width*height);
	this->depthPlayer.resize(width*height);

	this->colorCameraFrameBuffer.resize(width * 4 * height); //32bit640*480 ARGB
	this->depthCameraFrameBuffer.resize(width * 2 * height); //16bit640*480

	asyncDevice = true; //񓯊
}

KinectSensor::~KinectSensor(){
	if(kinect != 0) {
		kinect->NuiShutdown();		//J
		kinect->Release();			//foCX[X
	}
}


void KinectSensor::start(){
	std::cout << "kinect starting...";
	try {
		//Kinect̏ݒ(J[摜gp)
		kinect->NuiInitialize(
			NUI_INITIALIZE_FLAG_USES_DEPTH_AND_PLAYER_INDEX
			| NUI_INITIALIZE_FLAG_USES_SKELETON
			| NUI_INITIALIZE_FLAG_USES_COLOR);

		streamHandles[0] = ::CreateEvent(NULL, TRUE, FALSE, NULL);
		streamHandles[1] = ::CreateEvent(NULL, TRUE, FALSE, NULL);
		streamHandles[2] = ::CreateEvent(NULL, TRUE, FALSE, NULL);

		//RGB camera init
		imageStreamHandle = ::CreateEvent(0, TRUE, FALSE, 0);
		kinect->NuiImageStreamOpen(NUI_IMAGE_TYPE_COLOR, /* NUI_IMAGE_TYPE_COLOR_YUV */
			cameraResolution,
			0, 2, 0, &imageStreamHandle);

		//depth camera init
		depthStreamHandle = ::CreateEvent(0, TRUE, FALSE, 0);
		kinect->NuiImageStreamOpen(NUI_IMAGE_TYPE_DEPTH_AND_PLAYER_INDEX,
			depthResolution,
			0, 2, 0, &depthStreamHandle);

		//XPg̗L
		//skeletonStreamHandle = ::CreateEvent(0, TRUE, FALSE, 0);
		//kinect->NuiSkeletonTrackingEnable(streamHandles[2], NUI_SKELETON_TRACKING_FLAG_SUPPRESS_NO_FRAME_DATA); //skeletonStreamHandle

		//t[XVCxg̍쐬
		if(!asyncDevice) {
			streamEvent = ::CreateEvent(0, TRUE, FALSE, 0);
			kinect->NuiSetFrameEndEvent(streamEvent, 0);
		}

		//zCgoX؂ꂻȂ؂
		HRESULT hr = kinect->NuiGetColorCameraSettings(&kinectSettings);
		if(hr == S_OK) {
			kinectSettings->SetAutoWhiteBalance(false);
		}

		//萔𑜓x̐l𓾂
		::NuiImageResolutionToSize(cameraResolution, width, height);
		::NuiImageResolutionToSize(depthResolution, depthWidth, depthHeight);
	} catch(const char* str) {
		throw std::runtime_error(str);
	}
	std::cout << "done." << std::endl;

}

void KinectSensor::stop(){

}

void KinectSensor::setCameraSwitch(bool rgb, bool depth){
	disableRGB = rgb;
	disableDepth = depth;
}
