// -*- C++ -*-
/*!
 * @file  RTC_Kinect_JointTracking.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include <Windows.h>
#include <NuiApi.h>

#include "RTC_Kinect_JointTracking.h"

// Module specification
// <rtc-template block="module_spec">
static const char* rtc_kinect_jointtracking_spec[] =
  {
    "implementation_id", "RTC_Kinect_JointTracking",
    "type_name",         "RTC_Kinect_JointTracking",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "ryo takeuchi",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTC_Kinect_JointTracking::RTC_Kinect_JointTracking(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_skeletonOut("dp_skeleton", m_skeleton)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_Kinect_JointTracking::~RTC_Kinect_JointTracking()
{
}



RTC::ReturnCode_t RTC_Kinect_JointTracking::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("dp_skeleton", m_skeletonOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // </rtc-template>

  jointTrackingMain.init();
  m_skeleton.data.length(60);

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onActivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RTC_Kinect_JointTracking::onExecute(RTC::UniqueId ec_id)
{
	jointTrackingMain.update();

	

	//

	memcpy(&(m_skeleton.data[0]), &(jointTrackingMain.skeleton[0]), sizeof(float) * 20 * 3);
	m_skeletonOut.write();

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_JointTracking::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void RTC_Kinect_JointTrackingInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtc_kinect_jointtracking_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTC_Kinect_JointTracking>,
                             RTC::Delete<RTC_Kinect_JointTracking>);
  }
  
};


