#include <iostream>
#include <sstream>
#include <array>
#include <opencv2/opencv.hpp>

#include <string>
#include <fstream>
#include <boost/tokenizer.hpp>
#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>

//o[W擾
#define CV_VERSION_STR CVAUX_STR(CV_MAJOR_VERSION) CVAUX_STR(CV_MINOR_VERSION) CVAUX_STR(CV_SUBMINOR_VERSION)
//rh[h
#ifdef _DEBUG
#define CV_EXT_STR "d.lib"
#else
#define CV_EXT_STR ".lib"
#endif

#pragma comment(lib, "opencv_core"		CV_VERSION_STR CV_EXT_STR)
#pragma comment(lib, "opencv_highgui"	CV_VERSION_STR CV_EXT_STR)
#pragma comment(lib, "opencv_imgproc"	CV_VERSION_STR CV_EXT_STR)
//#pragma comment(lib, "opencv_calib3d"	CV_VERSION_STR CV_EXT_STR)

#include "IntersectionDeterminator.h"

struct CalibrationData {
	cv::Point3d planeNormal, planePosition;
	std::vector<cv::Point2f> point;	//Lu[Vp̒n_
};

struct VirtualWallData {
	struct CalibrationData calibData;
	cv::Mat Homo;

	VirtualWallData(){
		calibData.point.resize(4);
	}
};


class Kinect_PointingMain {
private:
	std::vector<IntersectionDeterminator> interDeter;

	std::vector<struct VirtualWallData> wallList;

	bool intersection;
	int intersectDistance, intersectPlaneID;
	cv::Point2d intersectionPoint;

	void calcHomography(){
		std::vector<cv::Point2f> vwallPos;
		vwallPos.push_back(cv::Point2f(0.0, 0.0));
		vwallPos.push_back(cv::Point2f(0.0, 1.0));
		vwallPos.push_back(cv::Point2f(1.0, 1.0));
		vwallPos.push_back(cv::Point2f(1.0, 0.0));

		for(int i = 0; i < wallList.size(); i++) {
			//wallList[i].Homo = cv::findHomography(vwallPos, wallList[i].calibData.point);
			wallList[i].Homo = cv::getPerspectiveTransform(vwallPos, wallList[i].calibData.point);
		}
	}

public:
	Kinect_PointingMain(){
		
	}

	//t@Cݒǂ
	void load(std::string filepath){
		interDeter.clear();
		wallList.clear();

		struct VirtualWallData wallData;

		try {
			std::ifstream ifs;
			std::cout << filepath << "J܂" << std::endl;
			ifs.open(filepath, std::ios::in); // ǂݎpŊJ
			std::string str;
			char buf[200];
			cv::Point3d point3d;
			cv::Point point;

			boost::char_separator<char> sep("e ,"); //separator is space or comma.
			//boost::char_separator<char> sep(",", "", boost::keep_empty_tokens);

			typedef boost::tokenizer<boost::char_separator<char> > tokenizer1;

			int lineCount = 0;
			while(!ifs.eof()) {
				ifs.getline(buf, 200);//printf("[%d]%s\n",i,buf);
				if(strlen(buf) <= 1) {
					continue;
				}

				if(buf[0] == '*') continue;

				using boost::lexical_cast;
				str = boost::lexical_cast<std::string>(buf);
				tokenizer1 tok1(str, sep);
				int count = 0;

				if(lineCount % 6 == 0) {
					for(tokenizer1::iterator it = tok1.begin(); it != tok1.end(); ++it) {
						if(count % 3 == 0) wallData.calibData.planeNormal.x = lexical_cast<int>(*it);
						else if(count % 3 == 1) wallData.calibData.planeNormal.y = lexical_cast<int>(*it);
						else if(count % 3 == 2) wallData.calibData.planeNormal.z = lexical_cast<int>(*it);

						count++;
					}
				}
				else if(lineCount % 6 == 1) {
					for(tokenizer1::iterator it = tok1.begin(); it != tok1.end(); ++it) {
						if(count % 3 == 0) wallData.calibData.planePosition.x = lexical_cast<int>(*it);
						else if(count % 3 == 1) wallData.calibData.planePosition.y = lexical_cast<int>(*it);
						else if(count % 3 == 2) wallData.calibData.planePosition.z = lexical_cast<int>(*it);
						
						count++;
					}
				}
				else if(lineCount % 6 >= 2) {
					for(tokenizer1::iterator it = tok1.begin(); it != tok1.end(); ++it) {
						if(count % 2 == 0) wallData.calibData.point[lineCount-2].x = lexical_cast<int>(*it);
						else if(count % 2 == 1) wallData.calibData.point[lineCount-2].y = lexical_cast<int>(*it);
						
						count++;
					}

					//Xgɒǉ
					if(lineCount % 6 == 5) {
						wallList.push_back(wallData);
						interDeter.push_back(IntersectionDeterminator(wallData.calibData.planePosition, wallData.calibData.planeNormal));
					}
				}
				lineCount++;
			}

			ifs.close();
		} catch(std::exception &ex) {
			std::cerr << ex.what() << std::endl;
		}

		std::cout << "o^ς݂̃f[^: " << wallList.size() << std::endl;


		calcHomography(); //zOtBvZ
	}

	void update(cv::Point3d start, cv::Point3d end){
		int intersectCount = 0;

		//Sƌ
		for(int i = 0; i < interDeter.size(); i++) {
			if(interDeter[i].update(start, end)) {
				intersectCount += 1;
			}
		}

		//Ă߂I
		if(intersectCount == 0) intersection = false;
		else {
			intersection = true;

			//ŒZ̕ʂ
			double minDist = DBL_MAX;
			int nearPlaneID = -1;
			for(int i = 0; i < interDeter.size(); i++) {
				if(interDeter[i].isIntersect()) {
					if(interDeter[i].getDistance() < minDist) {
						nearPlaneID = i;
						minDist = interDeter[i].getDistance();
					}
				}
			}

			if(nearPlaneID != -1) {
				intersectPlaneID = nearPlaneID;
				intersectDistance = minDist;

				//ZoĂzOtBϊ
				cv::Point ip = conv3Dto2D(interDeter[intersectPlaneID].getIntersectionPoint(), interDeter[intersectPlaneID].getPlaneNormal());
				cv::Mat loc = cv::Mat(cv::Matx31d(ip.x, ip.y, 1));
				cv::Mat ret = wallList[intersectPlaneID].Homo.inv() * loc;

				intersectionPoint = cv::Point2d(ret.at<double>(0, 0) / ret.at<double>(2, 0), ret.at<double>(1, 0) / ret.at<double>(2, 0));
			}
		}
	}

	//ȂH
	cv::Point conv3Dto2D(cv::Point3d point3D, cv::Point3d normal){
		cv::Point retval = cv::Point(0, 0);

		if(normal.x == 1) {
			retval.x = point3D.y;
			retval.y = point3D.z;
		}
		else if(normal.x == -1) {
			retval.x = point3D.y;
			retval.y = point3D.z;
		}
		else if(normal.z == -1) {
			retval.x = point3D.x;
			retval.y = point3D.y;
		}
		else if(normal.z == 1) {
			retval.x = point3D.x;
			retval.y = point3D.y;
		}
		else if(normal.y == -1) {
			retval.x = point3D.x;
			retval.y = point3D.z;
		}
		else if(normal.y == 1) {
			retval.x = point3D.x;
			retval.y = point3D.z;
		}

		return retval;
	}


	//_(ϊ)𓾂
	cv::Point2d getIntersectionPoint(){
		return intersectionPoint;
	}

	//Ă邩ǂ𓾂
	bool isIntersect(){
		return intersection;
	}

	//o^Ă镽ʔԍ_	
	int getSurfaceID(){
		return intersectPlaneID;
	}
};