// -*- C++ -*-
/*!
 * @file  RTC_Kinect_Pointing.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "RTC_Kinect_Pointing.h"

// Module specification
// <rtc-template block="module_spec">
static const char* rtc_kinect_pointing_spec[] =
{
	"implementation_id", "RTC_Kinect_Pointing",
	"type_name", "RTC_Kinect_Pointing",
	"description", "ModuleDescription",
	"version", "1.0.0",
	"vendor", "ryo takeuchi",
	"category", "Category",
	"activity_type", "PERIODIC",
	"kind", "DataFlowComponent",
	"max_instance", "1",
	"language", "C++",
	"lang_type", "compile",
	""
};
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTC_Kinect_Pointing::RTC_Kinect_Pointing(RTC::Manager* manager)
// <rtc-template block="initializer">
: RTC::DataFlowComponentBase(manager),
m_startPointIn("dp_start", m_startPoint),
m_endPointIn("dp_end", m_endPoint),
m_userPositionIn("dp_position", m_userPosition),
m_pointOut("dp_point", m_point),
m_surfaceOut("dp_surface", m_surface),
m_pointingOut("dp_pointing", m_pointing)

// </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTC_Kinect_Pointing::~RTC_Kinect_Pointing()
{
}



RTC::ReturnCode_t RTC_Kinect_Pointing::onInitialize()
{
	// Registration: InPort/OutPort/Service
	// <rtc-template block="registration">
	// Set InPort buffers
	addInPort("dp_start", m_startPointIn);
	addInPort("dp_end", m_endPointIn);
	addInPort("dp_position", m_userPositionIn);

	// Set OutPort buffer
	addOutPort("dp_point", m_pointOut);
	addOutPort("dp_surface", m_surfaceOut);
	addOutPort("dp_pointing", m_pointingOut);

	// Set service provider to Ports

	// Set service consumers to Ports

	// Set CORBA Service Ports

	// </rtc-template>

	// <rtc-template block="bind_config">
	// </rtc-template>

	pointingMain.load("calibration.dat");

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onFinalize()
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onStartup(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onShutdown(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RTC_Kinect_Pointing::onActivated(RTC::UniqueId ec_id)
{
	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onDeactivated(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RTC_Kinect_Pointing::onExecute(RTC::UniqueId ec_id)
{
	if(m_startPointIn.isNew()) {
		m_startPointIn.read();
	}
	if(m_endPointIn.isNew()) {
		m_endPointIn.read();
	}
	if(m_userPositionIn.isNew()) {
		m_userPositionIn.read();
	}

	pointingMain.update(cv::Point3d(m_startPoint.x, m_startPoint.y, m_startPoint.z),
		cv::Point3d(m_endPoint.x, m_endPoint.y, m_endPoint.z));

	if(pointingMain.isIntersect()) {
		//wn_̏o
		cv::Point2d intersection = pointingMain.getIntersectionPoint();

		m_point.x = intersection.x;
		m_point.y = intersection.y;
		m_pointOut.write();

		//ʂ̏o
		m_surface.data = pointingMain.getSurfaceID();
		m_surfaceOut.write();
	}
	//wwĂ邩ǂ̃tO
	m_pointing.data = pointingMain.isIntersect();
	m_pointingOut.write();

	return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onAborting(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onError(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onReset(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onStateUpdate(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTC_Kinect_Pointing::onRateChanged(RTC::UniqueId ec_id)
{
return RTC::RTC_OK;
}
*/



extern "C"
{

	void RTC_Kinect_PointingInit(RTC::Manager* manager)
	{
		coil::Properties profile(rtc_kinect_pointing_spec);
		manager->registerFactory(profile,
			RTC::Create<RTC_Kinect_Pointing>,
			RTC::Delete<RTC_Kinect_Pointing>);
	}

};


